/*
 *  File-Roller
 *
 *  Copyright (C) 2004 Free Software Foundation, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Author: Paolo Bacchilega <paobac@cvs.gnome.org>
 *
 */

#include <config.h>
#include <string.h>
#include <glib/gi18n-lib.h>
#include <gio/gio.h>
#include <libnautilus-extension/nautilus-extension-types.h>
#include <libnautilus-extension/nautilus-file-info.h>
#include <libnautilus-extension/nautilus-menu-provider.h>
#include "nautilus-filecompress.h"


#include <stdio.h>
#include <stdlib.h>
#include <sys/wait.h>
#include <sys/types.h>


static GObjectClass *parent_class;
char *file_type=NULL;
char *name_after_cute = NULL;
gboolean DEBUG = FALSE;
char * replace(
    char const * const original, 
    char const * const pattern, 
    char const * const replacement
) {
  size_t const replen = strlen(replacement);
  size_t const patlen = strlen(pattern);
  size_t const orilen = strlen(original);

  size_t patcnt = 0;
  const char * oriptr;
  const char * patloc;

  // find how many times the pattern occurs in the original string
  for (oriptr = original; patloc = strstr(oriptr, pattern); oriptr = patloc + patlen)
  {
    patcnt++;
  }

  {
    // allocate memory for the new string
    size_t const retlen = orilen + patcnt * (replen - patlen);
    char * const returned = (char *) malloc( sizeof(char) * (retlen + 1) );

    if (returned != NULL)
    {
      // copy the original string, 
      // replacing all the instances of the pattern
      char * retptr = returned;
      for (oriptr = original; patloc = strstr(oriptr, pattern); oriptr = patloc + patlen)
      {
        size_t const skplen = patloc - oriptr;
        // copy the section until the occurence of the pattern
        strncpy(retptr, oriptr, skplen);
        retptr += skplen;
        // copy the replacement 
        strncpy(retptr, replacement, replen);
        retptr += replen;
      }
      // copy the rest of the string.
      strcpy(retptr, oriptr);
    }
    return returned;
  }
}

int  cut_source_string(char *source)
{	int i =0;
	for(i=0;i<strlen(source);){
		if((*(source+i)&0x80)){//中文&&!中文字符
			/*
			 if(((*source&0xff)==0xA1)&&((*(source+1)&0xff)==0xA3)){
				 i++;
				printf("..........get...source...%s....%s....\n",*source,*(source+1));
			}else{
		 		i=i+3;
			}
			*/
			i=i+3;
		}else{
			i++;
			}
		
		if(i>9){
			printf(".........i....=%d\n",i);
			return i;
		}
	}

}
char *get_file_name_cut(char *source){
	char *file_name = NULL; 
	int lenght=strlen(source);
	if(lenght>20){
		file_name = g_strconcat(g_strndup(source,cut_source_string(source)),"...",NULL);
//		file_name = g_strconcat(g_strndup(source,10),"...",NULL);
	}else{
		file_name = g_strdup(source);
		}
		gchar *tt = "__";
		char *unline = replace(file_name,"_",tt);
		return unline;
}
char *get_file_name_no_dail(char *source){
    
    if(file_type == NULL){
        return;
    }
//   printf("...file_type...not...null\n");
    char *find;
    if(source!=NULL){
        if(DEBUG){
                printf("...source...not...null\n");
                printf("...source=%s\n",source);
                printf("...file_type=%s\n",file_type);
        }
        find = g_strrstr(source,file_type);
    }
    if(find !=NULL){
        if(DEBUG){
                 printf("...find...not...null\n");
        }
        int len = strlen(source) -strlen(find);
        if(DEBUG){
                 printf(".......return....%s\n",g_strndup(source,len));
        }
        return g_strndup(source,len);
    }
     
     return ""; 

}

static void
extract_open_callback (NautilusMenuItem *item,
		     gpointer          user_data)
{
	GList            *files;
	NautilusFileInfo *file;
	char             *uri, *default_dir;
	GString          *cmd;

	files = g_object_get_data (G_OBJECT (item), "files");
	file = files->data;

	uri = nautilus_file_info_get_uri (file);
	default_dir = nautilus_file_info_get_parent_uri (file);

	cmd = g_string_new ("file-compress");
	g_string_append_printf (cmd,
				"  %s",
				g_shell_quote (uri));


	g_print ("EXEC: %s\n", cmd->str);

	g_spawn_command_line_async (cmd->str, NULL);

	g_string_free (cmd, TRUE);
	g_free (default_dir);
	g_free (uri);
}

static void
extract_to_callback (NautilusMenuItem *item,
		     gpointer          user_data)
{
	GList            *files;
	NautilusFileInfo *file;
	char             *uri, *default_dir;
	GString          *cmd;

	files = g_object_get_data (G_OBJECT (item), "files");
	file = files->data;

	uri = nautilus_file_info_get_uri (file);
	default_dir = nautilus_file_info_get_parent_uri (file);

	cmd = g_string_new ("file-compress");
	g_string_append_printf (cmd,
				" --default-dir=%s --extract %s",
				g_shell_quote (default_dir),
				g_shell_quote (uri));


	g_print ("EXEC: %s\n", cmd->str);


	g_spawn_command_line_async (cmd->str, NULL);

	g_string_free (cmd, TRUE);
	g_free (default_dir);
	g_free (uri);
}

static void
extract_name_callback (NautilusMenuItem *item,
		       gpointer          user_data)
{
	GList            *files, *scan;
	NautilusFileInfo *file;
	char             *dir,*file_name;
	GString          *cmd;

	files = g_object_get_data (G_OBJECT (item), "files");
	file = files->data;

	dir = nautilus_file_info_get_parent_uri (file);
	//	printf("....dir........%s\n",dir);
	cmd = g_string_new ("file-compress");
	//g_string_append_printf (cmd,"   --extract-to=");

	

	for (scan = files; scan; scan = scan->next) {
		NautilusFileInfo *file = scan->data;
		char             *uri;

		uri = nautilus_file_info_get_uri (file);
		
		g_string_append_printf (cmd, "  %s   --extract-to=", g_shell_quote (uri));
		g_free (uri);
	}
	file_name= g_strconcat(dir,"/",name_after_cute,NULL);
	g_string_append_printf (cmd,"%s",g_shell_quote(file_name));
	g_string_append_printf(cmd,"  --force");

	g_spawn_command_line_async (cmd->str, NULL);
	
//	printf("EXEC: %s\n", cmd->str);
	g_string_free (cmd, TRUE);
	g_free (dir);
}

/*
static void
extract_here_callback (NautilusMenuItem *item,
		       gpointer          user_data)
{
	GList            *files, *scan;
	NautilusFileInfo *file;
	char             *dir;
	GString          *cmd;

	files = g_object_get_data (G_OBJECT (item), "files");
	file = files->data;

	dir = nautilus_file_info_get_parent_uri (file);

	cmd = g_string_new ("file-compress");
	g_string_append_printf (cmd," --extract-here");

	g_free (dir);

	for (scan = files; scan; scan = scan->next) {
		NautilusFileInfo *file = scan->data;
		char             *uri;

		uri = nautilus_file_info_get_uri (file);
		g_string_append_printf (cmd, " %s", g_shell_quote (uri));
		g_free (uri);
	}

	g_spawn_command_line_async (cmd->str, NULL);
	
	g_print ("EXEC: %s\n", cmd->str);
	g_string_free (cmd, TRUE);
}
*/

void GetPath(char *pString,char *pCrrentDir) //解析文件路径
{
     while(*(pCrrentDir++)=*(pString++));
    while(*(--pCrrentDir)!='/');
    *pCrrentDir++='\'';
    *pCrrentDir='\0';
}



static void
extract_here_callback (NautilusMenuItem *item,
               gpointer          user_data)
{
    GList            *files, *scan;
    NautilusFileInfo *file;
    char             *dir,*file_name;
    GString          *cmd;

    files = g_object_get_data (G_OBJECT (item), "files");
    file = files->data;
    dir = nautilus_file_info_get_parent_uri (file);
    cmd = g_string_new ("file-compress");

    for (scan = files; scan; scan = scan->next)
    {
        NautilusFileInfo *file = scan->data;
        char             *uri;

        uri = nautilus_file_info_get_uri (file);
        g_string_append_printf (cmd, " %s  --extract-to=", g_shell_quote (uri));
        g_free (uri);
    }

    file_name= g_strconcat(dir,"/",name_after_cute,NULL);
    char buf[1024]={0};
    GetPath(g_shell_quote(file_name),buf);

    g_string_append_printf (cmd,"%s",buf);
    g_string_append_printf(cmd,"  --force");

    g_spawn_command_line_async (cmd->str, NULL);

    g_string_free (cmd, TRUE);
    g_free (dir);
}

static void
add_callback (NautilusMenuItem *item,
	      gpointer          user_data)
{
	GList            *files, *scan;
	NautilusFileInfo *file;
	char             *uri, *dir;
	GString          *cmd;

	files = g_object_get_data (G_OBJECT (item), "files");
	file = files->data;

	uri = nautilus_file_info_get_uri (file);
	dir = g_path_get_dirname (uri);

	cmd = g_string_new ("file-compress");
	g_string_append_printf (cmd," --default-dir=%s --add", g_shell_quote (dir));


	g_free (dir);
	g_free (uri);

	for (scan = files; scan; scan = scan->next) {
		NautilusFileInfo *file = scan->data;

		uri = nautilus_file_info_get_uri (file);
		g_string_append_printf (cmd, " %s", g_shell_quote (uri));
		g_free (uri);
	}

	g_spawn_command_line_async (cmd->str, NULL);

	g_string_free (cmd, TRUE);
}

static void
add_zip_callback (NautilusMenuItem *item,
	      gpointer          user_data)
{


	GList            *files, *scan;
	NautilusFileInfo *file;
	char             *uri, *dir,*zip,*first_uri;
	GString          *cmd;

	files = g_object_get_data (G_OBJECT (item), "files");
	file = files->data;

	first_uri = nautilus_file_info_get_uri (file);
	dir = g_path_get_dirname (first_uri);

	cmd = g_string_new ("file-compress");	

	for (scan = files; scan; scan = scan->next) {
		NautilusFileInfo *file = scan->data;

		uri = nautilus_file_info_get_uri (file);
		g_string_append_printf (cmd, " %s  ", g_shell_quote (uri));
		g_free (uri);
	}
	
        zip = g_strconcat(first_uri,".zip",NULL);
        g_string_append_printf(cmd,"--add-to  %s",g_shell_quote(zip));
	g_spawn_command_line_async (cmd->str, NULL);
	g_string_free (cmd, TRUE);
	g_free(zip);
	g_free(first_uri);
        g_free (dir);
	
}

/*static void 
install_rar_command_callback(NautilusMenuItem *item,
		gpointer user_data)
{
        

	g_spawn_command_line_async ("/usr/share/file-compress/icons/install_rar.sh",NULL);


}*/
int check_cmd_rtcode(char * cmd)   //sxh add
{
    pid_t status;
    status = system(cmd);
    if (-1 == status)
        return -1;
    else
    {
        if (WIFEXITED(status))
        {
            if (0 == WEXITSTATUS(status))
                return 0;
            else
                return -2;
        }
        else
            return -3;
    }
}

void testclose(GtkWidget *window, gpointer data)
{
    GtkWidget *window1=(GtkWidget *)data;
    gtk_widget_destroy(window1);
}

void install_rar_shell(GtkWidget *window, gpointer data)
{
    g_spawn_command_line_async ("/usr/share/file-compress/icons/install_rar.sh",NULL);
    GtkWidget *window1=(GtkWidget *)data;
    gtk_widget_destroy(window1);
}



void getlicencestr(char *licence)
{
    char *str = "\n \
以下内容为用win.rar有限公司进行开发的RAR软件的使\n\
用许可--任何安装，访问，或者用任何其他方式使用该软\n\
件的用户需遵守此协议。\n\n\n\
 1， 亚历山大L.Roshal为该软件的拥有者和持有人，拥\n\
有该软件的著作权。作为许可证的发行人，拥有该软件的\n\
许可权和全世界的承载独家使用权，包括任何形式的复制\n\
，发行和向公众发布软件的权利都归win.rar有限公司所\n\
有，公司地址为：Schumannstr,17,10117柏林，德国。\n\n\n\
 2， 软件以先试后买的方式发布，这意味着任何人可以\n\
在最多40天的测试期间免费使用该软件，试用期过后，用\n\
户必须购买一个许可证才能继续使用该软件。\n\n\n\
 3， 软件试用版可以自由发布，需遵守以下协议，并且\n\
提供的软件包不能以任何方式进行修改。\n\n\
   a，未经许可，任何人不得将软件按独立的功能发布。\n\n\
   b，未经许可，该软件的试用版本内不得夹杂其他任\n\
   何软件包。\n\
   c，试用版本内不能包含用于破解的秘钥，或秘钥生\n\
   成器。\n\n\n\
 4，该软件的试用版可以显示注册提示对话框。根据不同\n\
的软件版本和配置，对话框可以包含一个预定义的文本和\n\
本地加载链接，或从因特网加载的网页。此网页可以包含\n\
许可说明或其他许可人选择的内容，包括广告。当点击这\n\
个网页，软件置灰传送技术上需要的参数，通过HTTP协议\n\
在浏览器中成功打开一个所需的网页。\n\n\n\
 5，使用原版软件时，没有任何明示或暗示的许可。您使\n\
用时自己承担相应的风险。无论作者，许可人，还是代理\n\
商都不对数据丢失，损坏，或其他形式的利益损失承担赔\n\
偿责任。\n\n\n\
 6， 以下为对软件发出的2种基本类型的许可证：\n\n\
  用户只能在一台电脑购买一个许可证。\n\n\
  家庭用户可以使用他们的许可证应用与所有的电脑和移\n\
  动设备上（包括U盘，移动硬盘等）。只要是许可证所\n\
  有者的财产即可。\n\n\
  商业用户需要在每台电脑或移动设备上使用一个许可证\n\n\n\
 7， 除此之外，除了牌照的成本，没有额外的许可费，\n\
用RAR压缩的文件的创建和发布，卷，自解压格式压缩文\n\
或自解压卷。许可证的拥有者可以使用该软件产生的文件\n\
并分发这些文件，无任何附加费。\n\n\n\
 8， 许可的软件不可以出租或租赁，但可以永久整体转\n\
让，如果接受者同意该许可的条款。\n\n\n\
 9， 要购买许可证，请阅读提供的文件order.htm，里\n\
面包含该软件的详细信息。\n\n\n\
 10， 除本协议规定的内容外，您不得使用，复制，模\n\
仿，克隆，出租，租赁，出售，修改，反编译，反汇编\n\
反向工程，任何未经授权的使用将导致本许可的立即自\n\
动终止，并可能导致刑事和/或民事责任。该软件可使用\n\
由第三方开发和/或受版权保护的组件。请阅读“致谢”\n\
帮助acknow.txt。\n\n\n\
 11， 本许可协议在德国法律下享有完全解释的权利。\n\
如果您是个商人，位于德国柏林注册win.rar有限公司的\n\
法院具有专属管辖权，因本许可证而引起的任何争议，协\n\
议或他的有效性由此法院受理。\n\n\n\
 12， 安装并使用该软件即表示您接收这些条款和许可条\n\
件。如果你不与许可提出的条件达成一致，您必须从你的\n\
储存设备中删除所有软件文件并停止使用该软件。" ;
    strcpy(licence,str);
}

void rar_licence_dialog()
{

    GtkWidget *window;
   GtkWidget *okbutton;
   GtkWidget *cancelbutton;
   GtkWidget *label;
   GtkWidget *headlabel;
   GtkWidget *scrolled_window;
   char listr[4096]={0};
   getlicencestr(listr);


   window = gtk_window_new( GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title(GTK_WINDOW(window), "最终用户许可协议");
   // gtk_window_set_resizable(GTK_WINDOW(window),FALSE);


   okbutton = gtk_button_new_with_label("确定安装");
   cancelbutton = gtk_button_new_with_label("取消");
   scrolled_window=gtk_scrolled_window_new(NULL,NULL); /*创建滚动窗口构件*/
   gtk_widget_set_size_request(scrolled_window,380,285);
   headlabel = gtk_label_new("安装rar需要遵守以下协议:              ");


   label = gtk_label_new(listr);

   gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window),label);/*将文本视图构件加入滚动窗口*/
   gtk_widget_show (label);

   gtk_window_set_default_size(GTK_WINDOW(window), 380, 380);
   gtk_window_set_resizable(GTK_WINDOW(window),FALSE);
   gtk_widget_set_size_request(GTK_WIDGET(headlabel), 300, 40);   //大小
   gtk_widget_set_size_request(GTK_WIDGET(okbutton), 100, 40);   //大小
   gtk_widget_set_size_request(GTK_WIDGET(cancelbutton), 100, 40);

   GtkWidget  *pWidgetFixed = gtk_fixed_new();
   gtk_fixed_put( GTK_FIXED(pWidgetFixed), headlabel, 0, 0 );  //位置
   gtk_fixed_put( GTK_FIXED(pWidgetFixed), okbutton, 170, 350 );  //位置
   gtk_fixed_put( GTK_FIXED(pWidgetFixed), cancelbutton, 280, 350 );
   gtk_fixed_put( GTK_FIXED(pWidgetFixed), scrolled_window, 0, 50 );
   gtk_container_add( GTK_CONTAINER(window), pWidgetFixed );
   gtk_container_set_border_width(GTK_CONTAINER(window),10);

   g_signal_connect(okbutton, "clicked",
                    G_CALLBACK(install_rar_shell), window);

   g_signal_connect(cancelbutton, "clicked",
                    G_CALLBACK(testclose), window);
   g_signal_connect(window, "destroy",
                    G_CALLBACK(gtk_widget_destroy), window);
   gtk_window_set_position(GTK_WINDOW(window),GTK_WIN_POS_CENTER_ALWAYS);
   gtk_widget_show_all(window);

}

static void 
install_rar_command_callback(NautilusMenuItem *item,
                             gpointer user_data)       //sxh add
{
    int result=0;
    if(check_cmd_rtcode("apt-cache search all|grep '\\brar\\b'")!=0)
    {
        GtkWidget *dialog;
        GtkWidget      *window=user_data;
        dialog = gtk_message_dialog_new(window,
                                        GTK_DIALOG_DESTROY_WITH_PARENT,
                                        GTK_MESSAGE_ERROR,
                                        GTK_BUTTONS_OK,
                                        _("Not find in source"));
        gtk_window_set_title(GTK_WINDOW(dialog), _("Error"));
        gtk_dialog_run(GTK_DIALOG(dialog));
        gtk_widget_destroy(dialog);
    }
    else
    {
        rar_licence_dialog();
    }
}


static void
add_rar_callback (NautilusMenuItem *item,
	      gpointer          user_data)
{
	
	GList            *files, *scan;
	NautilusFileInfo *file;
	char             *uri, *dir,*zip,*first_uri;
	GString          *cmd;

	files = g_object_get_data (G_OBJECT (item), "files");
	file = files->data;

	first_uri = nautilus_file_info_get_uri (file);
	dir = g_path_get_dirname (first_uri);

	cmd = g_string_new ("file-compress");	

	for (scan = files; scan; scan = scan->next) {
		NautilusFileInfo *file = scan->data;

		uri = nautilus_file_info_get_uri (file);
		g_string_append_printf (cmd, " %s   ", g_shell_quote (uri));
		g_free (uri);
	}
	
        zip = g_strconcat(first_uri,".rar",NULL);
        g_string_append_printf(cmd," --add-to  %s",g_shell_quote(zip));
	g_spawn_command_line_async (cmd->str, NULL);
	
	//printf(".......add..to..rar....%s\n",cmd->str);
	g_string_free (cmd, TRUE);
	g_free(zip);
	g_free(first_uri);
        g_free (dir);
}
static void
add_7z_callback (NautilusMenuItem *item,
	      gpointer          user_data)
{
	
	GList            *files, *scan;
	NautilusFileInfo *file;
	char             *uri, *dir,*zip,*first_uri;
	GString          *cmd;

	files = g_object_get_data (G_OBJECT (item), "files");
	file = files->data;

	first_uri = nautilus_file_info_get_uri (file);
	dir = g_path_get_dirname (first_uri);

	cmd = g_string_new ("file-compress");	

	for (scan = files; scan; scan = scan->next) {
		NautilusFileInfo *file = scan->data;

		uri = nautilus_file_info_get_uri (file);
		g_string_append_printf (cmd, " %s ", g_shell_quote (uri));
		g_free (uri);
	}
	
        zip = g_strconcat(first_uri,".7z",NULL);
        g_string_append_printf(cmd," --add-to  %s",g_shell_quote(zip));
	g_spawn_command_line_async (cmd->str, NULL);
	g_string_free (cmd, TRUE);
	g_free(zip);
	g_free(first_uri);
        g_free (dir);
}
static struct {
	char     *mime_type;
	gboolean  is_compressed;
	char     *file_type;
} archive_mime_types[] = {
		{ "application/x-7z-compressed", TRUE,".7z" },
		{ "application/x-7z-compressed-tar", TRUE,".tar.7z" },
		{ "application/x-ace", TRUE , ".ace"},
		{ "application/x-alz", TRUE ,".alz"},
		{ "application/x-ar", TRUE , ".ar"},
		{ "application/x-arj", TRUE,".arj" },
		{ "application/x-bzip", TRUE ,".bz2"},
		{ "application/x-bzip-compressed-tar", TRUE,".tar.bz2" },
		{ "application/x-bzip1", TRUE,".bz" },
		{ "application/x-bzip1-compressed-tar", TRUE ,".tar.bz"},
		{ "application/vnd.ms-cab-compressed", TRUE,".cab" },
		{ "application/x-cbr", TRUE ,".cbr"},
		{ "application/x-cbz", TRUE,".cbz" },
		{ "application/x-cd-image", FALSE,".iso" },
		{ "application/x-compress", TRUE ,".Z"},
		{ "application/x-compressed-tar", TRUE ,".tar.gz"},
		{ "application/x-cpio", TRUE,".cpio" },
		{ "application/x-deb", TRUE ,".deb"},
		{ "application/x-ear", TRUE,".ear" },
		{ "application/x-ms-dos-executable", FALSE,".exe" },
		{ "application/x-gtar", FALSE ,".gz"},
		{ "application/x-gzip", TRUE,".gz" },
		{ "application/x-gzpostscript", TRUE ,".gz"},
		{ "application/x-java-archive", TRUE ,".jar"},
		{ "application/x-lha", TRUE,".lzh" },
		{ "application/x-lhz", TRUE, ".lz" },
		{ "application/x-lzip", TRUE ,".lz"},
		{ "application/x-lzip-compressed-tar", TRUE,".tar.lz" },
		{ "application/x-lzma", TRUE,"lzma" },
		{ "application/x-lzma-compressed-tar", TRUE,".tar.lzma" },
		{ "application/x-lzop", TRUE, ".lzo"},
		{ "application/x-lzop-compressed-tar", TRUE ,".tar.lzo"},
		{ "application/x-ms-wim", TRUE,".wim" },
		{ "application/x-rar", TRUE,".rar" },
//		{ "application/x-rar-compressed", ".tar.rar" },
		{ "application/x-rpm", TRUE ,".rpm"},
		{ "application/x-rzip", TRUE ,".rz"},
		{ "application/x-tar", FALSE ,".tar"},
		{ "application/x-tarz", TRUE ,".tar.Z"},
		{ "application/x-stuffit", TRUE,".sit" },
		{ "application/x-war", TRUE ,".war"},
		{ "application/x-xz", TRUE ,".xz"},
		{ "application/x-xz-compressed-tar", TRUE ,".tar.xz"},
		{ "application/x-zip", TRUE,".zip"},
		{ "application/x-zip-compressed", TRUE ,".zip"},
		{ "application/x-zoo", TRUE ,".zoo"},
		{ "application/zip", TRUE ,".zip"},
		{ "multipart/x-zip", TRUE ,".zip"},
		{ NULL, FALSE }
};


typedef struct {
      gboolean is_archive;
      gboolean is_derived_archive;
      gboolean is_compressed_archive;
} FileMimeInfo;


static FileMimeInfo
get_file_mime_info (NautilusFileInfo *file)
{
        if(DEBUG){
                printf("....get_file_mine_info.....call...call...\n");
        }
	FileMimeInfo file_mime_info;
	int          i;

	file_mime_info.is_archive = FALSE;
	file_mime_info.is_derived_archive = FALSE;
	file_mime_info.is_compressed_archive = FALSE;

	for (i = 0; archive_mime_types[i].mime_type != NULL; i++)
		if (nautilus_file_info_is_mime_type (file, archive_mime_types[i].mime_type)) {
			char *mime_type;
			char *content_type_mime_file;
			char *content_type_mime_compare;

			mime_type = nautilus_file_info_get_mime_type (file);

			content_type_mime_file = g_content_type_from_mime_type (mime_type);
			content_type_mime_compare = g_content_type_from_mime_type (archive_mime_types[i].mime_type);

			file_mime_info.is_archive = TRUE;
			file_mime_info.is_compressed_archive = archive_mime_types[i].is_compressed;
			if(file_type==NULL){
			        file_type = archive_mime_types[i].file_type;//tonihu add 
			         if(DEBUG){
			                printf("....file_type=NULL.....get..is===%s...\n",file_type);
			         }
			}else{
			        file_type == NULL;
			        file_type = archive_mime_types[i].file_type;
			        if(DEBUG){
			                 printf("....file_type isnot!=NULL.....get..is===%s...\n",file_type);
			        }
			}
			if ((content_type_mime_file != NULL) && (content_type_mime_compare != NULL))
				file_mime_info.is_derived_archive = ! g_content_type_equals (content_type_mime_file, content_type_mime_compare);

			g_free (mime_type);
			g_free (content_type_mime_file);
			g_free (content_type_mime_compare);

			return file_mime_info;
		}

	return file_mime_info;
}


static gboolean
unsupported_scheme (NautilusFileInfo *file)
{
	gboolean  result = FALSE;
	GFile    *location;
	char     *scheme;

	location = nautilus_file_info_get_location (file);
	scheme = g_file_get_uri_scheme (location);

	if (scheme != NULL) {
		const char *unsupported[] = { "trash", "computer", NULL };
		int         i;

		for (i = 0; unsupported[i] != NULL; i++)
			if (strcmp (scheme, unsupported[i]) == 0)
				result = TRUE;
	}

	g_free (scheme);
	g_object_unref (location);

	return result;
}


static GList *
nautilus_fr_get_file_items (NautilusMenuProvider *provider,
			    GtkWidget            *window,
			    GList                *files)
{
	GList    *items = NULL;
	GList    *scan;
	gboolean  can_write = TRUE;
	gboolean  one_item;
	gboolean  one_archive = FALSE;
	gboolean  one_derived_archive = FALSE;
	gboolean  one_compressed_archive = FALSE;
	gboolean  all_archives = TRUE;
	gboolean  all_archives_derived = TRUE;
	gboolean  all_archives_compressed = TRUE;
	gboolean  show_compress_and_extract=FALSE;
	char *file_name = NULL;
	char *file_name_changed =NULL;
	if (files == NULL)
		return NULL;

	if (unsupported_scheme ((NautilusFileInfo *) files->data))
			return NULL;

	for (scan = files; scan; scan = scan->next) {
		NautilusFileInfo *file = scan->data;
		FileMimeInfo      file_mime_info;
		file_name = nautilus_file_info_get_name(file);
		if(file_name_changed!=NULL){
			file_name_changed =NULL;
			file_name_changed = get_file_name_cut(file_name);
		}else{
			file_name_changed = get_file_name_cut(file_name);
		}

		file_mime_info = get_file_mime_info (file);

                if(name_after_cute!=NULL){
                        name_after_cute = NULL;
                        name_after_cute = get_file_name_no_dail(file_name);
                }else{
                        name_after_cute = get_file_name_no_dail(file_name);
                }		
		if (all_archives && ! file_mime_info.is_archive)
			all_archives = FALSE;

		if (all_archives_compressed && file_mime_info.is_archive && ! file_mime_info.is_compressed_archive)
			all_archives_compressed = FALSE;

		if (all_archives_derived && file_mime_info.is_archive && ! file_mime_info.is_derived_archive)
			all_archives_derived = FALSE;

		if (can_write) {
			NautilusFileInfo *parent;

			parent = nautilus_file_info_get_parent_info (file);
 			can_write = nautilus_file_info_can_write (parent);
		}
		if(g_file_get_path(nautilus_file_info_get_location(file))==NULL){
			show_compress_and_extract=TRUE;
		}
	}
	if(show_compress_and_extract){
			return NULL;
	}
	/**/

	one_item = (files != NULL) && (files->next == NULL);
	one_archive = one_item && all_archives;
	one_derived_archive = one_archive && all_archives_derived;
	one_compressed_archive = one_archive && all_archives_compressed;

//解压选项
	if (all_archives && can_write) {
	//用压缩工具打开
	        NautilusMenuItem *item_open;
		item_open = nautilus_menu_item_new ("NautilusFr::extract_open",
					       _("open with filecompress"),
					       _("Extract the selected archive"),
					       "extract-archive");
		g_signal_connect (item_open,
				  "activate",
				  G_CALLBACK (extract_open_callback),
				  provider);
		g_object_set_data_full (G_OBJECT (item_open),
					"files",
					nautilus_file_info_list_copy (files),
					(GDestroyNotify) nautilus_file_info_list_free);

		items = g_list_append (items, item_open);
	//解压到。。。。
	        NautilusMenuItem *item_extract_to;
		item_extract_to = nautilus_menu_item_new ("NautilusFr::extract_to",
					       _("Extract To..."),
					       _("Extract the selected archive"),
					       "extract-archive");
		g_signal_connect (item_extract_to,
				  "activate",
				  G_CALLBACK (extract_to_callback),
				  provider);
		g_object_set_data_full (G_OBJECT (item_extract_to),
					"files",
					nautilus_file_info_list_copy (files),
					(GDestroyNotify) nautilus_file_info_list_free);

		items = g_list_append (items, item_extract_to);

	//解压到当前文件夹
		NautilusMenuItem *item_extract_here;

		item_extract_here = nautilus_menu_item_new ("NautilusFr::extract_current",
					       _("Extract Here"),
					       /* Translators: the current position is the current folder */
					       _("Extract the selected archive to the current position"),
					       "extract-archive");
		g_signal_connect (item_extract_here,
				  "activate",
				  G_CALLBACK (extract_here_callback),
				  provider);
		g_object_set_data_full (G_OBJECT (item_extract_here),
					"files",
					nautilus_file_info_list_copy (files),
					(GDestroyNotify) nautilus_file_info_list_free);

		items = g_list_append (items, item_extract_here);
		
	//解压到当前文件夹名......
		NautilusMenuItem *item_extract_name;
		item_extract_name = nautilus_menu_item_new ("NautilusFr::extract_name",
		                                g_strconcat(_("Extract To"),get_file_name_cut(name_after_cute),"/",NULL),
					   //    _("Extract to"+name_after_cute+"/"),
					       /* Translators: the current position is the current folder */
					       _("Extract the selected archive to the current position"),
					       "extract-archive");
		g_signal_connect (item_extract_name,
				  "activate",
				  G_CALLBACK (extract_name_callback),
				  provider);
		g_object_set_data_full (G_OBJECT (item_extract_name),
					"files",
					nautilus_file_info_list_copy (files),
					(GDestroyNotify) nautilus_file_info_list_free);

		items = g_list_append (items, item_extract_name);		
		
	}

//压缩选项   
	if (! one_compressed_archive || one_derived_archive) {
	//添加到压缩软件
		NautilusMenuItem *item_add;

		item_add = nautilus_menu_item_new ("NautilusFr::add_normal",
					       _("add to tar"),
					       _("Create a compressed archive with the selected objects"),
					       "add-files-to-archive");
		g_signal_connect (item_add,
				  "activate",
				  G_CALLBACK (add_callback),
				  provider);
		g_object_set_data_full (G_OBJECT (item_add),
					"files",
					nautilus_file_info_list_copy (files),
					(GDestroyNotify) nautilus_file_info_list_free);

		items = g_list_append (items, item_add);
		
	//添加到“文件名.zip"
		NautilusMenuItem *item_add_zip;

		item_add_zip = nautilus_menu_item_new ("NautilusFr::add_zip",
					       g_strconcat(_("add to tar"),"\"" ,file_name_changed,".zip","\"",NULL),
					       _("Create a compressed archive with the selected objects"),
					       "add-files-to-archive");
		g_signal_connect (item_add_zip,
				  "activate",
				  G_CALLBACK (add_zip_callback),
				  provider);
		g_object_set_data_full (G_OBJECT (item_add_zip),
					"files",
					nautilus_file_info_list_copy (files),
					(GDestroyNotify) nautilus_file_info_list_free);

		items = g_list_append (items, item_add_zip);	
	//otherMenu
	

					
	        NautilusMenuItem *item_other;

		item_other = nautilus_menu_item_new ("NautilusFr::Other",
					       _("Other command"),
					       _("Extract the selected archive"),
					       "extract-archive");

                NautilusMenu *otherMenu = nautilus_menu_new();
                nautilus_menu_item_set_submenu(item_other,otherMenu);
                NautilusMenuItem *item_opentt;
		item_opentt = nautilus_menu_item_new ("NautilusFr::extract_open",
					       _("open with filecompress"),
					       _("Extract the selected archive"),
					       "extract-archive");
		g_signal_connect (item_opentt,
				  "activate",
				  G_CALLBACK (extract_open_callback),
				  provider);
		g_object_set_data_full (G_OBJECT (item_opentt),
					"files",
					nautilus_file_info_list_copy (files),
					(GDestroyNotify) nautilus_file_info_list_free);
					
		GFile *gfile =NULL;
		gfile= g_file_new_for_path("/usr/bin/rar");
		NautilusMenuItem *Item_rar;
               if(g_file_query_exists(gfile,NULL)){		
                Item_rar = nautilus_menu_item_new ("NautilusFr::add_rar",
					       g_strconcat(_("add to tar"),"\"",file_name_changed,".rar","\"",NULL),
					       _("rar the selected archive"),
					       "extract-archive");
		g_signal_connect (Item_rar,
				  "activate",
				  G_CALLBACK (add_rar_callback),
				  provider);
		g_object_set_data_full (G_OBJECT (Item_rar),
					"files",
					nautilus_file_info_list_copy (files),
					(GDestroyNotify) nautilus_file_info_list_free);
		}else{
			printf("........file_is..node....exists..........\n");
		   Item_rar =  nautilus_menu_item_new ("NautilusFr::add_rar",
					       _("add to tar install rar"),
					       _("rar the selected archive"),
					       "/usr/share/file-compress/icons/hicolor/16x16/actions/add-files-to-archive-compress.png");
			g_signal_connect (Item_rar,
				  "activate",
				  G_CALLBACK (install_rar_command_callback),
                  window);
			g_object_set_data_full (G_OBJECT (Item_rar),
					"files",
					nautilus_file_info_list_copy (files),
					(GDestroyNotify) nautilus_file_info_list_free);
		
		
		
		}			
                NautilusMenuItem *Item_7z = nautilus_menu_item_new ("NautilusFr::add_7z",
					       g_strconcat(_("add to 7z"),"\"",file_name_changed,".7z","\"",NULL),
					       _("MD5 the selected archive"),
					       "/usr/share/icons/CDOS-X/actions/16/menu-extract.png");
		g_signal_connect (Item_7z,
				  "activate",
				  G_CALLBACK (add_7z_callback),
				  provider);
		g_object_set_data_full (G_OBJECT (Item_7z),
					"files",
					nautilus_file_info_list_copy (files),
					(GDestroyNotify) nautilus_file_info_list_free);
                NautilusMenuItem *Item_email = nautilus_menu_item_new ("NautilusFr::add_email",
					       g_strconcat(_("add to tar"),file_name_changed,".zip",NULL),
					       _("right the selected archive"),
					       "menu-extract");
//		nautilus_menu_append_item(otherMenu,item_opentt);			       
	        nautilus_menu_append_item(otherMenu,Item_rar);
	        nautilus_menu_append_item(otherMenu,Item_7z);
	      //  nautilus_menu_append_item(otherMenu,Item_email);
	
		g_object_set_data_full (G_OBJECT (item_other),
					"files",
					nautilus_file_info_list_copy (files),
					(GDestroyNotify) nautilus_file_info_list_free);

		items = g_list_append (items, item_other);	
		
	}

	return items;
}


static void
nautilus_fr_menu_provider_iface_init (NautilusMenuProviderIface *iface)
{
	iface->get_file_items = nautilus_fr_get_file_items;
}


static void
nautilus_fr_instance_init (NautilusFr *fr)
{
}


static void
nautilus_fr_class_init (NautilusFrClass *class)
{
	parent_class = g_type_class_peek_parent (class);
}


static GType fr_type = 0;


GType
nautilus_fr_get_type (void)
{
	return fr_type;
}


void
nautilus_fr_register_type (GTypeModule *module)
{
	static const GTypeInfo info = {
		sizeof (NautilusFrClass),
		(GBaseInitFunc) NULL,
		(GBaseFinalizeFunc) NULL,
		(GClassInitFunc) nautilus_fr_class_init,
		NULL,
		NULL,
		sizeof (NautilusFr),
		0,
		(GInstanceInitFunc) nautilus_fr_instance_init,
	};

	static const GInterfaceInfo menu_provider_iface_info = {
		(GInterfaceInitFunc) nautilus_fr_menu_provider_iface_init,
		NULL,
		NULL
	};

	fr_type = g_type_module_register_type (module,
					       G_TYPE_OBJECT,
					       "NautilusFileCompress",
					       &info, 0);

	g_type_module_add_interface (module,
				     fr_type,
				     NAUTILUS_TYPE_MENU_PROVIDER,
				     &menu_provider_iface_info);
}
