/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  File-Roller
 *
 *  Copyright (C) 2008, 2012 Free Software Foundation, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <gtk/gtk.h>
#include <config.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <gio/gio.h>
#include "file-utils.h"
#include "fr-init.h"
#include "fr-new-archive-dialog.h"
#include "glib-utils.h"
#include "gtk-utils.h"
#include "preferences.h"


#define GET_WIDGET(x)		(_gtk_builder_get_widget (self->priv->builder, (x)))
#define MEGABYTE		(1024 * 1024)
#define ARCHIVE_ICON_SIZE	48

struct _FrNewArchiveDialogPrivate {
	GSettings  *settings;
	GtkBuilder *builder;
	int        *supported_types;
	GHashTable *supported_ext;
	gboolean    can_encrypt;
	gboolean    can_encrypt_header;
	gboolean    can_create_volumes;
	GFile      *original_file;
};
/*
char *sizeName[] = {
_(512KB   - 论坛附件 小),
_(1MB      -  论坛附件 中),
_(2MB      -  论坛附件 大),
_(50MB    -  上传微博     ),
_(700MB  -   CD刻录      ),
_(4481MB - DVD刻录     ),
_(8151MB - 双层DVD刻录)
}
*/
char *sizeName[] = {
{"512KB   -Forum small attachment "},
{"1MB      -  Forum attachment"},
{"2MB      -  Forum accessories"},
{"50MB    -  Upload micro-blog"},
{"700MB  -  CD burning "},
{"4481MB - DVD burning "},
{"8151MB - DVD dual"}
};
char *compression[] = {
	{"Very Fast"},
	{"Fast     "},
	{"Normal   "},
	{"Maximum  "},
};
G_DEFINE_TYPE (FrNewArchiveDialog, fr_new_archive_dialog, GTK_TYPE_DIALOG)

char *filename_entry_save ;
void screen_changed(GtkWidget *widget, GdkScreen *old_screen, gpointer userdata)
{
    /* To check if the display supports alpha channels, get the visual */
    GdkScreen *screen = gtk_widget_get_screen(widget);
    GdkVisual *visual = gdk_screen_get_rgba_visual(screen);
    if (!visual)
    {
        printf("Your screen does not support alpha channels!\n");
        visual = gdk_screen_get_system_visual(screen);
    }
    else
    {
        printf("Your screen supports alpha channels!\n");
    }

    gtk_widget_set_visual(widget, visual);
}

 cairo_surface_t *
 scale_surface (cairo_surface_t *old_surface, int old_width, int old_height,int new_width, int new_height)
 {
  
    cairo_surface_t  *new_surface = cairo_surface_create_similar(old_surface,CAIRO_CONTENT_COLOR_ALPHA,new_width,new_height);
    cairo_t *cr = cairo_create (new_surface);

    /* Scale *before* setting the source surface (1) */
    cairo_scale (cr, (double)new_width / old_width, (double)new_height / old_height);
    cairo_set_source_surface (cr, old_surface, 0, 0);

    /* To avoid getting the edge pixels blended with 0 alpha, which would 
     * occur with the default EXTEND_NONE. Use EXTEND_PAD for 1.2 or newer (2)
      */
    cairo_pattern_set_extend (cairo_get_source(cr), CAIRO_EXTEND_REFLECT); 

     /* Replace the destination with the source instead of overlaying */
    cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);

    /* Do the actual drawing */
    cairo_paint (cr);
   
    cairo_destroy (cr);

    return new_surface;
 }

 
static gboolean draw(GtkWidget *widget, cairo_t *cr, gpointer userdata)
{
 gint root_width,root_height,image_width,image_height;
 
 GdkWindow *root;
 cairo_surface_t * surface =cairo_image_surface_create_from_png("/usr/share/file-compress/icons/new-archive-background.png");
  image_width = cairo_image_surface_get_width (surface);
  image_height = cairo_image_surface_get_height (surface);

   root_width=gtk_widget_get_allocated_width(widget);
   root_height=gtk_widget_get_allocated_height(widget);
   cairo_t *new_cr = gdk_cairo_create(gtk_widget_get_window(widget));
   

   cairo_surface_t *target;
  
   target = scale_surface(surface,image_width , image_height,root_width,root_height);
   cairo_set_source_rgba (new_cr, 0.5, 1.0, 0.50, 0.0); 


    cairo_set_operator (new_cr, CAIRO_OPERATOR_SOURCE);
    cairo_paint (new_cr);

   cairo_set_source_surface(new_cr,target,0,0);
        cairo_paint(new_cr);

    cairo_surface_destroy(surface);
    cairo_surface_destroy(target);
    cairo_destroy(new_cr);

    return FALSE;
}
static void
fr_new_archive_dialog_finalize (GObject *object)
{
	FrNewArchiveDialog *self;

	self = FR_NEW_ARCHIVE_DIALOG (object);

	_g_object_unref (self->priv->original_file);
	g_object_unref (self->priv->settings);
	g_object_unref (self->priv->builder);
	g_hash_table_unref (self->priv->supported_ext);

	G_OBJECT_CLASS (fr_new_archive_dialog_parent_class)->finalize (object);
}


static int
get_selected_format (FrNewArchiveDialog *self)
{
	return self->priv->supported_types[gtk_combo_box_get_active (GTK_COMBO_BOX (GET_WIDGET ("extension_comboboxtext")))];
}


static void
fr_new_archive_dialog_unmap (GtkWidget *widget)
{
	FrNewArchiveDialog *self;
	int                 n_format;

	self = FR_NEW_ARCHIVE_DIALOG (widget);

	g_settings_set_boolean (self->priv->settings, PREF_NEW_ENCRYPT_HEADER, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("encrypt_header_checkbutton"))));
	g_settings_set_int (self->priv->settings, PREF_NEW_VOLUME_SIZE, gtk_spin_button_get_value (GTK_SPIN_BUTTON (GET_WIDGET ("volume_spinbutton"))) * MEGABYTE);

	n_format = get_selected_format (self);
	g_settings_set_string (self->priv->settings, PREF_NEW_DEFAULT_EXTENSION, mime_type_desc[n_format].default_ext);

	GTK_WIDGET_CLASS (fr_new_archive_dialog_parent_class)->unmap (widget);
}


static void
fr_new_archive_dialog_class_init (FrNewArchiveDialogClass *klass)
{
	GObjectClass   *object_class;
	GtkWidgetClass *widget_class;

	g_type_class_add_private (klass, sizeof (FrNewArchiveDialogPrivate));

	object_class = G_OBJECT_CLASS (klass);
	object_class->finalize = fr_new_archive_dialog_finalize;

	widget_class = GTK_WIDGET_CLASS (klass);
	widget_class->unmap = fr_new_archive_dialog_unmap;
}


static void
fr_new_archive_dialog_init (FrNewArchiveDialog *self)
{
	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, FR_TYPE_NEW_ARCHIVE_DIALOG, FrNewArchiveDialogPrivate);
	self->priv->settings = g_settings_new (FILE_ROLLER_SCHEMA_NEW);
	self->priv->builder = NULL;
	self->priv->supported_ext = g_hash_table_new (g_str_hash, g_str_equal);
	self->priv->original_file = NULL;
}


static void
_fr_new_archive_dialog_update_sensitivity (FrNewArchiveDialog *self)
{
	gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (GET_WIDGET ("encrypt_header_checkbutton")), ! self->priv->can_encrypt_header);
	gtk_widget_set_sensitive (GET_WIDGET ("encrypt_header_checkbutton"), self->priv->can_encrypt_header);
	gtk_widget_set_sensitive (GET_WIDGET ("volume_spinbutton"), ! self->priv->can_create_volumes || gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("volume_checkbutton"))));
}


static void
password_entry_changed_cb (GtkEditable *editable,
			   gpointer     user_data)
{
	_fr_new_archive_dialog_update_sensitivity (FR_NEW_ARCHIVE_DIALOG (user_data));
}


static void
volume_toggled_cb (GtkToggleButton *toggle_button,
		   gpointer         user_data)
{
	_fr_new_archive_dialog_update_sensitivity (FR_NEW_ARCHIVE_DIALOG (user_data));
}

static void commbox2_changed_cb(GtkComboBox *combo_box,
				   gpointer    user_data)
{

	FrNewArchiveDialog *self = user_data;
	int                 n_format;


}
static void
extension_comboboxtext_changed_cb (GtkComboBox *combo_box,
				   gpointer     user_data)
{
	FrNewArchiveDialog *self = user_data;
	int                 n_format;
	GdkPixbuf          *icon_pixbuf;

	n_format = get_selected_format (self);
	self->priv->can_encrypt = mime_type_desc[n_format].capabilities & FR_ARCHIVE_CAN_ENCRYPT;
	gtk_widget_set_sensitive (GET_WIDGET ("password_entry"), self->priv->can_encrypt);
	gtk_widget_set_sensitive (GET_WIDGET ("password_label"), self->priv->can_encrypt);

	self->priv->can_encrypt_header = mime_type_desc[n_format].capabilities & FR_ARCHIVE_CAN_ENCRYPT_HEADER;
	gtk_widget_set_sensitive (GET_WIDGET ("encrypt_header_checkbutton"), self->priv->can_encrypt_header);

	self->priv->can_create_volumes = mime_type_desc[n_format].capabilities & FR_ARCHIVE_CAN_CREATE_VOLUMES;
//        if(n_format == 9 || n_format == 0)
//            self->priv->can_create_volumes = FALSE;
	gtk_widget_set_sensitive (GET_WIDGET ("combobox2"), self->priv->can_create_volumes);
	gtk_widget_set_sensitive (GET_WIDGET ("combobox1"), self->priv->can_create_volumes);
/*
	icon_pixbuf = _g_mime_type_get_icon (mime_type_desc[n_format].mime_type,
					     ARCHIVE_ICON_SIZE,
					     gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (self))));
	if (icon_pixbuf != NULL) {
		gtk_image_set_from_pixbuf (GTK_IMAGE (GET_WIDGET ("archive_icon")), icon_pixbuf);
		g_object_unref (icon_pixbuf);
	}
*/
	_fr_new_archive_dialog_update_sensitivity (self);
}


static char *
_g_path_remove_extension_if_archive (const char *filename)
{
	const char *ext;
	int         i;

	ext = _g_filename_get_extension (filename);
	if (ext == NULL)
		return g_strdup (filename);

	for (i = 0; file_ext_type[i].ext != NULL; i++) {
		if (strcmp (ext, file_ext_type[i].ext) == 0)
			return g_strndup (filename, strlen (filename) - strlen (ext))  ;
	}

	return g_strdup (filename);
}

static void radio_toggled4_cb(GtkWidget *widget,gpointer num){
	if(gtk_toggle_button_get_active(widget)){
		FrNewArchiveDialog *self = num;
		//printf("................4.............................\n");

		gtk_widget_hide(GET_WIDGET("other_options_expander"));
		gtk_widget_hide(GET_WIDGET("viewport3"));
		
		gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("extension_comboboxtext")), 17);
		char *old_name = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("filename_entry")));
		if(strlen(old_name)!=0){
		gchar *jiequ = g_strrstr(old_name, ".");
		char *new_name;
		if(jiequ !=NULL){
			int length = strlen(old_name)-strlen(jiequ);
			new_name = g_strndup(old_name, length);
		}else{
			new_name = old_name;
		}
	    char *new_name_zip= g_strconcat (new_name,".zip", NULL);

		gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("filename_entry")), new_name_zip);
	}
	}
}
static void radio_toggled5_cb(GtkWidget *widget,gpointer num){
	if(gtk_toggle_button_get_active(widget)){
		FrNewArchiveDialog *self = num;
		//printf("......................5.........................\n");
		
		gtk_widget_hide(GET_WIDGET("other_options_expander"));
		gtk_widget_hide(GET_WIDGET("viewport3"));
		
		gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("extension_comboboxtext")), 0);
		char *old_name = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("filename_entry")));
		if(strlen(old_name)!=0){
		gchar *jiequ = g_strrstr(old_name, ".");
		char *new_name;
		if(jiequ !=NULL){
			int length = strlen(old_name)-strlen(jiequ);
			new_name = g_strndup(old_name, length);
		}else{
			new_name = old_name;
		}
		
		char *new_name_7z= g_strconcat (new_name ,".7z", NULL);
		gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("filename_entry")), new_name_7z);
		}
	}
}
static void radio_toggled6_cb(GtkWidget *widget,gpointer num){
	if(gtk_toggle_button_get_active(widget)){
		FrNewArchiveDialog *self = num;
		gtk_widget_show(GET_WIDGET("other_options_expander"));
		gtk_widget_show(GET_WIDGET("viewport3"));
	
		

		char *old_name = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("filename_entry")));
		if(strlen(old_name)!=0){
		gchar *jiequ = g_strrstr(old_name, ".");
		char *new_name;
		if(jiequ !=NULL){
			int length = strlen(old_name)-strlen(jiequ);
			new_name = g_strndup(old_name,  length);
		}else{
			new_name = old_name;
		}
		gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("filename_entry")), new_name);
		}//printf("......................6.........................\n");

	}
}

static gint button_press_event(GtkWidget * widget,GdkEventButton * event, gpointer data)
{
	if (event->button == 1)  
	    {  
		gtk_window_begin_move_drag(GTK_WINDOW(gtk_widget_get_toplevel(widget)), event->button, event->  
	x_root, event->y_root, event->time);  
	    }  
        return TRUE;
}

void  new_archive_close_clicked(GtkWidget * widget,gpointer user_data)
	{
		//gtk_window_close(GTK_WINDOW(gtk_widget_get_toplevel(widget)));
        gtk_widget_destroy(GTK_WINDOW(gtk_widget_get_toplevel(widget)));
	}



	
static void
_fr_new_archive_dialog_construct (FrNewArchiveDialog *self,
				  GtkWindow          *parent,
				  FrNewArchiveAction  action,
				  GFile              *folder,
				  const char         *default_name,
				  GFile              *original_file,
				  const char * title)
{
	char *active_extension;
	int   active_extension_idx;
	int   i;
	GtkWidget * new_archive_create;
	printf("..............default_name.....=...%s......folder.....=...%s\n",default_name,g_file_get_path(folder));
	gtk_window_set_transient_for (GTK_WINDOW (self), parent);
	gtk_window_set_resizable (GTK_WINDOW (self), FALSE);
	gtk_container_set_border_width (GTK_CONTAINER (self), 5);

	self->priv->builder = _gtk_builder_new_from_resource ("new-archive-dialog.ui");
	if (self->priv->builder == NULL)
		return;

	_g_object_unref (self->priv->original_file);
	self->priv->original_file = _g_object_ref (original_file);

	gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (self))), GET_WIDGET ("content"));


//sunqi
	GtkWidget * new_archive_cancel =  gtk_dialog_add_button (GTK_DIALOG (self), "", GTK_RESPONSE_CANCEL);
        gtk_widget_set_name(GTK_WIDGET(new_archive_cancel),"new-archive-cancel");
	switch (action) {
	case FR_NEW_ARCHIVE_ACTION_NEW_MANY_FILES:
		self->priv->supported_types = create_type;
		new_archive_create = gtk_dialog_add_button (GTK_DIALOG (self), "", GTK_RESPONSE_OK);
		gtk_widget_set_name(GTK_WIDGET(new_archive_create),"new-archive-create");
		break;
	case FR_NEW_ARCHIVE_ACTION_NEW_SINGLE_FILE:
		self->priv->supported_types = single_file_save_type;
		new_archive_create = gtk_dialog_add_button (GTK_DIALOG (self), "", GTK_RESPONSE_OK);
		gtk_widget_set_name(GTK_WIDGET(new_archive_create),"new-archive-create");
		break;
	case FR_NEW_ARCHIVE_ACTION_SAVE_AS:
		self->priv->supported_types = save_type;
		gtk_dialog_add_button (GTK_DIALOG (self), _GTK_LABEL_SAVE, GTK_RESPONSE_OK);
		break;
	}
	gtk_dialog_set_default_response (GTK_DIALOG (self), GTK_RESPONSE_OK);
	gtk_widget_set_name(GTK_WIDGET(self),"new-archive-dialog");
	//sunqi 
	gtk_window_set_decorated (GTK_WINDOW(self),FALSE);
	GtkWidget * new_archive_title = _gtk_builder_get_widget (self->priv->builder, "label4");
	gtk_widget_set_name(GTK_WIDGET(new_archive_title),"new-archive-title");
	gtk_label_set_text(GTK_LABEL(new_archive_title),title);
	GtkImage * new_archive_close = _gtk_builder_get_widget (self->priv->builder, "image2");
	gtk_widget_set_name(GTK_WIDGET(new_archive_close),"progress_close");
	GtkImage * new_archive_title_img = _gtk_builder_get_widget (self->priv->builder, "image1");
	gtk_widget_set_name(GTK_WIDGET(new_archive_title_img),"progress-title-img");
	GtkWidget * new_archive_filename = _gtk_builder_get_widget (self->priv->builder, "archive_label");
	gtk_widget_set_name(GTK_WIDGET(new_archive_filename),"new-archive-location");
	GtkWidget * new_archive_location = _gtk_builder_get_widget (self->priv->builder, "location_label");
	gtk_widget_set_name(GTK_WIDGET(new_archive_location),"new-archive-filename");
	GtkViewport * new_archive_titlebar = _gtk_builder_get_widget (self->priv->builder, "viewport1");
	gtk_widget_set_name(GTK_WIDGET(new_archive_titlebar),"new-archive-titlebar");
	GtkViewport * new_archive_setting = _gtk_builder_get_widget (self->priv->builder, "viewport3");
	gtk_widget_set_name(GTK_WIDGET(new_archive_setting),"new-archive-setting");
	
	
	GtkViewport * archive_titlebar = _gtk_builder_get_widget (self->priv->builder, "viewport4");
	GtkEventBox * new_archive_close_eventbox = _gtk_builder_get_widget (self->priv->builder, "eventbox1");
	gtk_widget_add_events(GTK_WINDOW(archive_titlebar),GDK_BUTTON_PRESS_MASK);
	gtk_widget_add_events(GTK_WIDGET(new_archive_close_eventbox), GDK_BUTTON_PRESS_MASK);


	g_signal_connect(G_OBJECT(new_archive_close_eventbox), "button_press_event",
		   G_CALLBACK(new_archive_close_clicked), self);
   
        g_signal_connect(G_OBJECT(archive_titlebar), "button_press_event",
                   G_CALLBACK(button_press_event),archive_titlebar);

	sort_mime_types_by_extension (self->priv->supported_types);
	//tonihu add 
//add radio button

	GtkRadioButton *button4 = _gtk_builder_get_widget(self->priv->builder,"radiobutton4");
	GtkRadioButton *button5 = _gtk_builder_get_widget(self->priv->builder,"radiobutton5");
	GtkRadioButton *button6 = _gtk_builder_get_widget(self->priv->builder,"radiobutton6");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button4), TRUE);
	g_signal_connect(button4,
			  "toggled",
			  G_CALLBACK (radio_toggled4_cb),
			self);
	g_signal_connect(button5,
			  "toggled",
			  G_CALLBACK (radio_toggled5_cb),
			  self);
	g_signal_connect(button6,
			  "toggled",
			  G_CALLBACK (radio_toggled6_cb),
			  self);

	/* Set widgets data. */

	if (default_name != NULL) {
		char *default_name_no_ext;

		default_name_no_ext = _g_path_remove_extension_if_archive (default_name);
		filename_entry_save = default_name_no_ext ;
		char *new_name = g_strconcat (filename_entry_save,".zip", NULL);
		gtk_entry_set_text (GTK_ENTRY (GET_WIDGET ("filename_entry")), new_name);

		//g_free (default_name_no_ext);
	}
	gtk_widget_grab_focus (GET_WIDGET ("filename_entry"));
	gtk_editable_select_region (GTK_EDITABLE (GET_WIDGET ("filename_entry")), 0, -1);

	if (folder == NULL)
		folder = _g_file_get_home ();
	gtk_file_chooser_set_current_folder_file (GTK_FILE_CHOOSER (GET_WIDGET ("parent_filechooserbutton")), folder, NULL);
//压缩 格式
	gtk_expander_set_expanded (GTK_EXPANDER (GET_WIDGET ("other_options_expander")), FALSE);

	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("encrypt_header_checkbutton")),
				      g_settings_get_boolean (self->priv->settings, PREF_NEW_ENCRYPT_HEADER));

	gtk_spin_button_set_value (GTK_SPIN_BUTTON (GET_WIDGET ("volume_spinbutton")),
				   (double) g_settings_get_int (self->priv->settings, PREF_NEW_VOLUME_SIZE) / MEGABYTE);

	active_extension = g_settings_get_string (self->priv->settings, PREF_NEW_DEFAULT_EXTENSION);
	active_extension_idx = 0;
	//tonihu need changed
	for (i = 0; self->priv->supported_types[i] != -1; i++) {

		if (strcmp (".zip", mime_type_desc[self->priv->supported_types[i]].default_ext) == 0){
			active_extension_idx = i;
		
		}
		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (GET_WIDGET ("extension_comboboxtext")),
					        mime_type_desc[self->priv->supported_types[i]].default_ext);
	}
	gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("extension_comboboxtext")), active_extension_idx);
	g_free (active_extension);

	gtk_widget_set_vexpand (GET_WIDGET ("other_options_expander"), FALSE);
//压缩大小
        /*
	for(i=0;i<6;i++){
			gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (GET_WIDGET ("combobox2")),
					        sizeName[i]);
			printf("...............sizeName........................%s\n",sizeName[i]);
	}
	gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("combobox2")), 0);
//tonihu add 
//压缩方式
		for(i=0;i<4;i++){
			gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (GET_WIDGET ("combobox1")),
					        compression[i]);
			printf("...............compression........................%s\n",compression[i]);
	}
*/
        gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("combobox2")), 7);
	gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("combobox1")), 0);

	gtk_widget_hide(GET_WIDGET("other_options_expander"));
	gtk_widget_hide(GET_WIDGET("viewport3"));
//add end

	_fr_new_archive_dialog_update_sensitivity (self);
	extension_comboboxtext_changed_cb (GTK_COMBO_BOX (GET_WIDGET ("extension_comboboxtext")), self);
//end 
//压缩方式


	_gtk_entry_use_as_password_entry (GTK_ENTRY (GET_WIDGET ("password_entry")));

	/* Set the signals handlers. */

        gtk_widget_set_app_paintable(GTK_WINDOW(self), TRUE);
//tonihu changed for bug big
	screen_changed(GTK_WINDOW(self), NULL, NULL);
        g_signal_connect(G_OBJECT(GTK_WINDOW(self)), "draw", G_CALLBACK(draw), NULL);
        
        
	g_signal_connect (GET_WIDGET ("password_entry"),
			  "changed",
			  G_CALLBACK (password_entry_changed_cb),
			  self);
	g_signal_connect (GET_WIDGET ("volume_checkbutton"),
			  "toggled",
			  G_CALLBACK (volume_toggled_cb),
			  self);
	g_signal_connect (GET_WIDGET ("extension_comboboxtext"),
			  "changed",
			  G_CALLBACK (extension_comboboxtext_changed_cb),
			  self);
	/*
	g_signal_conncet(GET_WIDGET("combobox2"),
			  "changed",
			  G_CALLBACK (commbox2_changed_cb),
			  self);
	*/
}


GtkWidget *
fr_new_archive_dialog_new (const char         *title,
			   GtkWindow          *parent,
			   FrNewArchiveAction  action,
			   GFile              *folder,
			   const char         *default_name,
			   GFile              *original_file)
{
	FrNewArchiveDialog *self;

	self = g_object_new (FR_TYPE_NEW_ARCHIVE_DIALOG, "title", title, NULL);
	gtk_window_set_title (GTK_WINDOW (parent), _("Archive Manager"));
	_fr_new_archive_dialog_construct (self, parent, action, folder, default_name, original_file,title);

	return (GtkWidget *) self;
}


GFile *
fr_new_archive_dialog_get_file (FrNewArchiveDialog  *self,
			        const char         **mime_type)
{
	const char *basename;
	int         n_format;
	char       *basename_ext;
	GFile      *parent;
	GFile      *file;
	GError     *error = NULL;
	const char *file_mime_type;
	GFileInfo  *parent_info;
	GtkWidget  *dialog;

	/* Check whether the user entered a valid archive name. */

	basename = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("filename_entry")));
	if ((basename == NULL) || (*basename == '\0')) {
		GtkWidget *d;

		d = _gtk_error_dialog_new (GTK_WINDOW (self),
					   GTK_DIALOG_MODAL,
					   NULL,
					   _("Could not create the archive"),
					   "%s",
					   _("You have to specify an archive name."));
		gtk_window_set_title (GTK_WINDOW (d), _("Archive Manager"));

		gtk_dialog_run (GTK_DIALOG (d));
		gtk_widget_destroy (GTK_WIDGET (d));

		return NULL;
	}
        if(strlen(basename) > 255)
        {
               GtkWidget *d;

                d = _gtk_error_dialog_new (GTK_WINDOW (self),
                                           GTK_DIALOG_MODAL,
                                           NULL,
                                           _("Could not create the archive"),
                                           "%s",
                                           _("File name is too long."));
                gtk_window_set_title (GTK_WINDOW (d), _("Archive Manager"));

                gtk_dialog_run (GTK_DIALOG (d));
                gtk_widget_destroy (GTK_WIDGET (d));

                return NULL;

        }


	/* file */

	n_format = get_selected_format (self);
	parent = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (GET_WIDGET ("parent_filechooserbutton")));
	if (parent == NULL) {
		GtkWidget *d;

		d = _gtk_error_dialog_new (GTK_WINDOW (self),
					   GTK_DIALOG_MODAL,
					   NULL,
					   _("Could not create the archive"),
					   "%s",
					   _("You have to specify an archive name."));
                gtk_window_set_title (GTK_WINDOW (d), _("Archive Manager"));

		gtk_dialog_run (GTK_DIALOG (d));
		gtk_widget_destroy (GTK_WIDGET (d));

		return NULL;
	}
	
	
	if(!gtk_toggle_button_get_active(GET_WIDGET("radiobutton6"))){
		if(gtk_toggle_button_get_active(GET_WIDGET("radiobutton4"))&&g_strrstr(basename, ".zip")==NULL){
			basename_ext = g_strconcat (basename,".zip", NULL);
		}else if(gtk_toggle_button_get_active(GET_WIDGET("radiobutton5"))&&g_strrstr(basename, ".7z")==NULL){
			basename_ext = g_strconcat (basename,".7z", NULL);
		}else{
			basename_ext = g_strconcat (basename, NULL);
		}
		file = g_file_get_child_for_display_name (parent, basename_ext, &error);
	}else{
		basename_ext = g_strconcat (basename, mime_type_desc[n_format].default_ext, NULL);
		file = g_file_get_child_for_display_name (parent, basename_ext, &error);
	}
	
	
	
	


	if (file == NULL) {
		dialog = _gtk_error_dialog_new (GTK_WINDOW (self),
						GTK_DIALOG_MODAL,
						NULL,
						_("Could not create the archive"),
						"%s",
						error->message);
                gtk_window_set_title (GTK_WINDOW (dialog), _("Archive Manager"));

		gtk_dialog_run (GTK_DIALOG (dialog));

		gtk_widget_destroy (GTK_WIDGET (dialog));
		g_error_free (error);
		g_free (basename_ext);
		g_object_unref (parent);

		return NULL;
	}

	g_free (basename_ext);

	/* mime type */

	file_mime_type = mime_type_desc[n_format].mime_type;
	if (mime_type != NULL)
		*mime_type = file_mime_type;

	/* check permissions */

	parent_info = g_file_query_info (parent,
				         (G_FILE_ATTRIBUTE_ACCESS_CAN_READ ","
				          G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE ","
				          G_FILE_ATTRIBUTE_ACCESS_CAN_EXECUTE","
				          G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME),
				         0,
				         NULL,
				         &error);

	g_object_unref (parent);

	if (error != NULL) {
		g_warning ("Failed to get permission for extraction dir: %s", error->message);

		g_clear_error (&error);
		g_object_unref (parent_info);
		g_object_unref (file);

		return NULL;
	}

	if (! g_file_info_get_attribute_boolean (parent_info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE)) {
		dialog = _gtk_error_dialog_new (GTK_WINDOW (self),
						GTK_DIALOG_MODAL,
						NULL,
						_("Could not create the archive"),
						"%s",
						_("You don't have permission to create an archive in this folder"));
		gtk_window_set_title (GTK_WINDOW (dialog), _("Archive Manager"));

		gtk_dialog_run (GTK_DIALOG (dialog));

		gtk_widget_destroy (GTK_WIDGET (dialog));
		g_object_unref (parent_info);
		g_object_unref (file);

		return NULL;
	}

	/* check whehter the file is equal to the original file */

	if ((self->priv->original_file != NULL) && (g_file_equal (file, self->priv->original_file))) {
		dialog = _gtk_error_dialog_new (GTK_WINDOW (self),
						GTK_DIALOG_MODAL,
						NULL,
						_("Could not create the archive"),
						"%s",
						_("New name is the same as old one, please type other name."));
		gtk_window_set_title (GTK_WINDOW (dialog), _("Archive Manager"));

		gtk_dialog_run (GTK_DIALOG (dialog));

		gtk_widget_destroy (GTK_WIDGET (dialog));
		g_object_unref (parent_info);
		g_object_unref (file);

		return NULL;
	}

	/* overwrite confirmation */

	if (g_file_query_exists (file, NULL)) {
		char     *filename;
		char     *message;
		char     *secondary_message;
		gboolean  overwrite;

		filename = _g_file_get_display_basename (file);
		message = g_strdup_printf (_("A file named \"%s\" already exists.  Do you want to replace it?"), filename);
		secondary_message = g_strdup_printf (_("The file already exists in \"%s\".  Replacing it will overwrite its contents."), g_file_info_get_display_name (parent_info));
		dialog = _gtk_message_dialog_new (GTK_WINDOW (self),
						  GTK_DIALOG_MODAL,
						  _GTK_ICON_NAME_DIALOG_QUESTION,
						  message,
						  secondary_message,
						  _GTK_LABEL_CANCEL, GTK_RESPONSE_CANCEL,
						  _("_Replace"), GTK_RESPONSE_OK,
						  NULL);
		overwrite = gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK;

		gtk_widget_destroy (dialog);
		g_free (secondary_message);
		g_free (message);
		g_free (filename);

		if (overwrite) {
		        //g_file_delete (file, NULL, &error);
			if (error != NULL) {
				dialog = _gtk_error_dialog_new (GTK_WINDOW (self),
								GTK_DIALOG_MODAL,
								NULL,
								_("Could not delete the old archive."),
								"%s",
								error->message);
				gtk_window_set_title (GTK_WINDOW (dialog), _("Archive Manager"));

				gtk_dialog_run (GTK_DIALOG (dialog));

				gtk_widget_destroy (GTK_WIDGET (dialog));
				g_error_free (error);
				g_object_unref (parent_info);
				g_object_unref (file);

				return NULL;
			}
		}
		else
			g_clear_object (&file);
	}

	g_object_unref (parent_info);

	return file;
}


const char *
fr_new_archive_dialog_get_password (FrNewArchiveDialog *self)
{
	const char *password = NULL;
	int         n_format;

	n_format = get_selected_format (self);

	if (mime_type_desc[n_format].capabilities & FR_ARCHIVE_CAN_ENCRYPT)
		password = (char*) gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("password_entry")));

	return password;
}


gboolean
fr_new_archive_dialog_get_encrypt_header (FrNewArchiveDialog *self)
{
	gboolean encrypt_header = FALSE;
	int      n_format;

	n_format = get_selected_format (self);

	if (mime_type_desc[n_format].capabilities & FR_ARCHIVE_CAN_ENCRYPT) {
		const char *password = gtk_entry_get_text (GTK_ENTRY (GET_WIDGET ("password_entry")));
		if (password != NULL) {
			if (strcmp (password, "") != 0) {
			//	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (GET_WIDGET ("encrypt_header_checkbutton")),TRUE);//TONIHU ADD
				if (mime_type_desc[n_format].capabilities & FR_ARCHIVE_CAN_ENCRYPT_HEADER)
			//		encrypt_header = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("encrypt_header_checkbutton")));
					encrypt_header = TRUE;
			}
		}
	}

	return encrypt_header;
}

FrCompression    
fr_new_archive_dialog_get_compression(FrNewArchiveDialog *self)
{
	FrCompression   compression = 50;
	int   n_format;

	n_format = get_selected_format (self);
	if ((mime_type_desc[n_format].capabilities & FR_ARCHIVE_CAN_CREATE_VOLUMES)&&gtk_toggle_button_get_active(GET_WIDGET("radiobutton6"))){
			
		int index = gtk_combo_box_get_active (GTK_COMBO_BOX (GET_WIDGET ("combobox2")));
		switch (index){
		case 0:
			compression =FR_COMPRESSION_VERY_FAST; break;
		case 1:
			compression =FR_COMPRESSION_FAST; break;
		case 2:
			compression =FR_COMPRESSION_NORMAL; break;
		case 3:
			compression =FR_COMPRESSION_MAXIMUM; break;
		}
	}
	return compression;
}
gboolean
fr_new_archive_dialog_get_compress_backend(FrNewArchiveDialog *self){
	return gtk_toggle_button_get_active(GET_WIDGET("checkbutton6"));
}
gboolean        
fr_new_archive_dialog_get_is_delete_source  (FrNewArchiveDialog  *self){
	return gtk_toggle_button_get_active(GET_WIDGET("checkbutton1"));	

}

int
fr_new_archive_dialog_get_volume_size (FrNewArchiveDialog *self)
{
	guint volume_size = 0;
	int   n_format;

	n_format = get_selected_format (self);

	if ((mime_type_desc[n_format].capabilities & FR_ARCHIVE_CAN_CREATE_VOLUMES)&&gtk_toggle_button_get_active(GET_WIDGET("radiobutton6")))
	{
		
		int index = gtk_combo_box_get_active (GTK_COMBO_BOX (GET_WIDGET ("combobox2")));
		switch (index){
		case 0:
			volume_size =524288; break;
		case 1:
			volume_size =1048576;break;
		case 2:
			volume_size =1048576; break;
		case 3:
			volume_size =52428800; break;
		case 4:
			volume_size =734003200; break;
		case 5:
			volume_size =4698669056; break;
		case 6:
			volume_size =8546942976; break;

		}
	}

	return volume_size;
}

